"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
var jsonpath_1 = require("jsonpath");
var operators_1 = require("./operators");
var utils_1 = require("./utils");
var getReferenceValueInInput = function (root_input, reference, indexes) {
    if (typeof reference !== 'string') {
        throw new Error('Invalid Reference');
    }
    if (reference.includes('[$index]')) {
        indexes.forEach(function (index) {
            reference = reference.replace('$index', "" + index);
        });
    }
    var results = jsonpath_1.query(root_input, reference.startsWith('[') ? "$" + reference : reference, 1);
    return results && results.length > 0 ? results[0] : null;
};
var recursivelyMatchValue = function (input, schema, root_input, indexes) {
    if (utils_1.isPrimitiveType(schema)) {
        if (utils_1.isPrimitiveType(input)) {
            return input != schema;
        }
        if (Array.isArray(input)) {
            return !input.some(function (v, i) { return !recursivelyMatchValue(v, schema, root_input, __spreadArray(__spreadArray([], indexes), [i])); });
        }
        if (typeof input === 'object') {
            return true;
        }
    }
    if (Array.isArray(input)) {
        if (Array.isArray(schema)) {
            return schema.some(function (sub_schema) {
                return !input.some(function (array_input, i) {
                    return !recursivelyMatchValue(array_input, sub_schema, root_input, __spreadArray(__spreadArray([], indexes), [
                        i,
                    ]));
                });
            });
        }
        var schema_ops = Object.entries(schema).filter(function (_a) {
            var key = _a[0];
            return operators_1.operator_keys.includes(key);
        });
        if (schema_ops.length > 0) {
            return schema_ops.some(function (_a) {
                var key = _a[0], value = _a[1];
                try {
                    return !operators_1.default[key](input, value);
                }
                catch (_b) {
                    return true;
                }
            });
        }
        return !input.some(function (v, i) { return !recursivelyMatchValue(v, schema, root_input, __spreadArray(__spreadArray([], indexes), [i])); });
    }
    if (typeof schema === 'object') {
        if (schema['$or'] && Array.isArray(schema['$or'])) {
            return !schema['$or'].some(function (condition_schema) {
                return matchJsonToSchema(input, condition_schema, root_input, indexes);
            });
        }
        if (schema['$ref']) {
            return recursivelyMatchValue(input, getReferenceValueInInput(root_input, schema['$ref'], indexes), root_input, indexes);
        }
        var schema_ops = Object.entries(schema).filter(function (_a) {
            var key = _a[0];
            return operators_1.operator_keys.includes(key);
        });
        if (schema_ops.length > 0) {
            return schema_ops.some(function (_a) {
                var key = _a[0], value = _a[1];
                if (typeof value === 'object' && value && value['$ref']) {
                    value = getReferenceValueInInput(root_input, value['$ref'], indexes);
                }
                try {
                    return !operators_1.default[key](input, value);
                }
                catch (_b) {
                    return true;
                }
            });
        }
        if (utils_1.isPrimitiveType(input)) {
            return true;
        }
        return !matchJsonToSchema(input, schema, root_input, indexes);
    }
    return true;
};
var matchJsonToSchema = function (input, schema, root_input, indexes) {
    if (indexes === void 0) { indexes = []; }
    try {
        if (typeof schema === 'object' && schema['$not']) {
            var result = matchJsonToSchema(input, schema['$not'], root_input, indexes);
            delete schema['$not'];
            if (Object.keys(schema).length === 0 || result) {
                return !result;
            }
        }
        if (!root_input) {
            root_input = input;
        }
        if (utils_1.isPrimitiveType(input) || Array.isArray(input)) {
            return !recursivelyMatchValue(input, schema, input, indexes);
        }
        if (typeof schema === 'object') {
            return !Object.entries(schema).some(function (_a) {
                var key = _a[0], schema = _a[1];
                if (key === '$or' && Array.isArray(schema)) {
                    return !schema.some(function (condition_schema) {
                        return matchJsonToSchema(input, condition_schema, root_input, indexes);
                    });
                }
                if (key === '$and' && Array.isArray(schema)) {
                    return schema.some(function (condition_schema) {
                        return !matchJsonToSchema(input, condition_schema, root_input, indexes);
                    });
                }
                if (!Array.isArray(input) &&
                    input[key] === undefined) {
                    var result = true;
                    if (typeof schema === 'object' && schema['$exist'] === false) {
                        result = false;
                    }
                    if (result) {
                        return true;
                    }
                }
                return recursivelyMatchValue(input[key], schema, root_input, indexes);
            });
        }
        return !recursivelyMatchValue(input, schema, input, indexes);
    }
    catch (e) {
        return false;
    }
};
exports.default = matchJsonToSchema;
