"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.operator_keys = void 0;
var utils_1 = require("./utils");
var operators = {
    $eq: function (v, compare) {
        v = utils_1.isPrimitiveType(v) ? v : JSON.stringify(v);
        compare = utils_1.isPrimitiveType(compare) ? compare : JSON.stringify(compare);
        return v == compare;
    },
    $neq: function (v, compare) {
        return !operators.$eq(v, compare);
    },
    $in: function (v, compare) {
        if (!utils_1.supportedType(compare, ['array', 'string']) ||
            !utils_1.supportedType(v, ['number', 'string', 'boolean', 'null'])) {
            throw new Error('Unsupported types for $in or $nin operator');
        }
        if (Array.isArray(compare)) {
            return compare.includes(v);
        }
        return v.includes(compare);
    },
    $nin: function (v, compare) {
        return !operators.$in(v, compare);
    },
    $startsWith: function (v, compare) {
        if (!utils_1.supportedType(v, ['string']) || !utils_1.supportedType(compare, ['string'])) {
            throw new Error('Unsupported types for $startsWith operator');
        }
        return v.startsWith(compare);
    },
    $endsWith: function (v, compare) {
        if (!utils_1.supportedType(v, ['string']) || !utils_1.supportedType(compare, ['string'])) {
            throw new Error('Unsupported types for $endsWith operator');
        }
        return v.endsWith(compare);
    },
    $gte: function (v, compare) {
        if (!utils_1.supportedType(v, ['string', 'number']) ||
            !utils_1.supportedType(compare, ['string', 'number'])) {
            throw new Error('Unsupported types for $gte operator');
        }
        return v >= compare;
    },
    $gt: function (v, compare) {
        if (!utils_1.supportedType(v, ['string', 'number']) ||
            !utils_1.supportedType(compare, ['string', 'number'])) {
            throw new Error('Unsupported types for $gt operator');
        }
        return v > compare;
    },
    $lte: function (v, compare) {
        if (!utils_1.supportedType(v, ['string', 'number']) ||
            !utils_1.supportedType(compare, ['string', 'number'])) {
            throw new Error('Unsupported types for $lte operator');
        }
        return v <= compare;
    },
    $lt: function (v, compare) {
        if (!utils_1.supportedType(v, ['string', 'number']) ||
            !utils_1.supportedType(compare, ['string', 'number'])) {
            throw new Error('Unsupported types for $lt operator');
        }
        return v < compare;
    },
    $exist: function (v, compare) {
        if (!utils_1.supportedType(compare, ['boolean'])) {
            throw new Error('Unsupported type for $exist operator');
        }
        if (compare === true && v !== undefined) {
            return true;
        }
        if (compare === false && v === undefined) {
            return true;
        }
        return false;
    },
};
exports.operator_keys = Object.keys(operators);
exports.default = operators;
